/*
 * eeprom.c
 *
 *  Created on: 25.01.2013
 *      Author: kalle
 */

#include <avr/io.h>

#include "main.h"
#ifdef DEBUG
	#include "uart.h"
#endif
//#ifdef DELAY
	#include <util/delay.h>
//#endif
#include "eeprom.h"
#include "i2cmaster.h"			// I2C interace


uint8_t _eeprom_init = 0;


void eeprom_init()
{
	uint8_t ret;

	ret = i2c_start(Dev24C512+I2C_WRITE);

	if (ret)
	{
		/* failed to issue start condition, possibly no device found */
#ifdef DEBUG
		usart_write("failed to issue start condition, possibly no device found." CR);
#endif
		_eeprom_init = 0;
	}
	else
	{
		_eeprom_init = 1;
	}

	i2c_stop();
}

uint8_t eeprom_read(uint16_t addr)
{
	if (_eeprom_init != 1)
		return 0;

	i2c_start_wait(Dev24C512+I2C_WRITE);
	//i2c_write( (addr/256) );
	//i2c_write( (addr%256) );
	i2c_write((addr>>8)&0xFF);
	i2c_write(addr&0xFF);
	i2c_rep_start(Dev24C512+I2C_READ);
	uint8_t ret = i2c_readNak();
	i2c_stop();

#ifdef DELAY
_delay_ms(DELAY);
#endif

	return ret;
}

void eeprom_write(uint16_t addr, uint8_t val)
{
	if (_eeprom_init != 1)
		return;

	/*while(i2c_start(Dev24C512+I2C_WRITE) != 0)
		i2c_stop();
	i2c_stop();*/

	i2c_start_wait(Dev24C512+I2C_WRITE);
	//i2c_write( (addr/256) );
	//i2c_write( (addr%256) );
	i2c_write((addr>>8)&0xFF);
	i2c_write(addr&0xFF);
	i2c_write(val);
	i2c_stop();

#ifdef DELAY
_delay_ms(DELAY);
#endif
}

void eeprom_write_inc(uint16_t* addr, uint8_t val)
{
	eeprom_write(*addr, val);
	(*addr) = eeprom_inc_addr((*addr) + 1);
}

uint8_t eeprom_read_inc(uint16_t* addr)
{
	uint8_t val = eeprom_read(*addr);
	(*addr) = eeprom_inc_addr((*addr) + 1);
	return val;
}


uint16_t eeprom_read_twice(uint16_t addr)
{
	uint8_t ret[2];

	/*if (_eeprom_init != 1)
		return 0x00;

	i2c_start_wait(Dev24C512+I2C_WRITE);

	ret[0] = i2c_write( (addr/256) );
	ret[0] = i2c_write( (addr%256) );
	i2c_rep_start(Dev24C512+I2C_READ);
	ret[1] = i2c_readAck(); // high byte
	ret[0] = i2c_readNak(); // low byte

	i2c_stop();

#ifdef DELAY
_delay_ms(DELAY);
#endif*/

	ret[1] = eeprom_read(addr);
	ret[0] = eeprom_read(eeprom_inc_addr(addr+1));

	return ret[0] + ret[1] * 100;
}

uint16_t eeprom_read_twice_inc(uint16_t* addr)
{
	uint16_t val = eeprom_read_twice(*addr);
	(*addr) = eeprom_inc_addr((*addr) + 2);
	return val;
}

void eeprom_write_twice(uint16_t addr, uint16_t val)
{
	uint8_t byte[2];

	byte[0] = val % 100; // low byte
	byte[1] = val / 100; // high byte

	/*i2c_start_wait(Dev24C512+I2C_WRITE);
	i2c_write( (addr/256) );
	i2c_write( (addr%256) );
	i2c_write(byte[1]);
	i2c_write(byte[0]);
	i2c_stop();

#ifdef DELAY
_delay_ms(DELAY);
#endif*/

	eeprom_write(addr, byte[1]);
	eeprom_write(eeprom_inc_addr(addr+1), byte[0]);
}

void eeprom_write_twice_inc(uint16_t* addr, uint16_t val)
{
	eeprom_write_twice(*addr, val);
	(*addr) = eeprom_inc_addr((*addr) + 2);
}

uint16_t eeprom_inc_addr(uint32_t addr)
{
	while (addr >= EEPROM_SIZE)
		addr -= EEPROM_SIZE;

	return (uint16_t)addr;
}
